/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.sinfo.flood_duration;

import java.util.List;

import org.apache.log4j.Logger;
import org.dive4elements.artifactdatabase.state.DefaultFacet;
import org.dive4elements.artifacts.Artifact;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.D4EArtifact;
import org.dive4elements.river.artifacts.model.Calculation;
import org.dive4elements.river.artifacts.model.CalculationResult;
import org.dive4elements.river.artifacts.sinfo.flood_duration.FloodDurationCalculationResult.Infrastructure;
import org.dive4elements.river.artifacts.states.DefaultState.ComputeType;
import org.dive4elements.river.exports.fixings.FixChartGenerator;
import org.dive4elements.river.jfree.RiverAnnotation;
import org.dive4elements.river.jfree.StickyAxisAnnotation;

/**
 * Facet to show Main Q Values.
 */
public class FloodDurationMainValuesQFacet extends DefaultFacet {

    private static final long serialVersionUID = 1L;

    /** Own log. */
    private static Logger log = Logger.getLogger(FloodDurationMainValuesQFacet.class);

    private final Infrastructure m_infrastructure;

    /** Trivial Constructor. */
    public FloodDurationMainValuesQFacet(final String name, final String description, final int facetIndex, final Infrastructure infrastructure) {
        this.description = description;
        this.name = name;
        this.index = facetIndex;
        this.m_infrastructure = infrastructure;
    }

    /**
     * Returns the data this facet requires.
     */
    @Override
    public Object getData(final Artifact artifact, final CallContext context) {

        log.debug("Get data for flood duration main value Q data");

        final D4EArtifact flys = (D4EArtifact) artifact;

        final CalculationResult res = (CalculationResult) flys.compute(context, ComputeType.ADVANCE, false);

        final FloodDurationCalculationResults data = (FloodDurationCalculationResults) res.getData();

        final double currentKm = FixChartGenerator.getCurrentKm(context);

        final FloodDurationCalculationResult result = data.getResults().get(0);

        final Calculation problems = new Calculation();

        final FacetCalculator calculator = new FacetCalculator(context);
        final List<StickyAxisAnnotation> annotations = calculator.calcMainValueQAnnotations(problems, currentKm, result);

        return new RiverAnnotation(this.description, annotations);
    }

    /**
     * Create a deep copy of this Facet.
     *
     * @return a deep copy.
     */
    @Override
    public FloodDurationMainValuesQFacet deepCopy() {
        final FloodDurationMainValuesQFacet copy = new FloodDurationMainValuesQFacet(this.name, this.description, this.index, this.m_infrastructure);
        copy.set(this);
        return copy;
    }
}